#!/bin/sh
#
# A script to automatically grab a kernel and build it with the specified configuration file.
#
# It will build a Debian package in DESTINATIONPATH using the source and config
# found in SOURCEPATH.
#

# Text appended to the kernel version
APPENDTOVERSION="-kernelbuilder"

# Where to find the kernel
SOURCEPATH=/mnt/host 

# Where to find any module tar files
MODULES=""

# This file can override the settings here
DEFAULTS="${SOURCEPATH}/kernelbuild-defaults"

# Where do we find the kernel configuration file
CONFIGFILE="${SOURCEPATH}/config"

# Where to place the final kernel package
DESTINATIONPATH=${SOURCEPATH}

# Where do we log the output
LOGPATH="${DESTINATIONPATH}/buildlog"

set -e

extract_tar () {
	if [ `echo ${1} | egrep '\.tar\.bz2$'` != "" ]; then
	tar xvjf ${1}
	elif [ `echo ${1} | egrep '\.tar\.gz$'` != "" ]; then
		tar xvzf ${1}
	elif [ `echo ${1} | egrep '\.tar$'` != "" ]; then
		tar xvf ${1}
	else
		echo "Cannot unpack ${1}"
		exit 1
	fi
}
	
if [ -r ${DEFAULTS} ]; then
	. ${DEFAULTS}
fi

# if we are logging
if [ "x${LOGPATH}" != "x" ]; then
	# and logging has not yet occurred, start it up
	if [ ! `echo $SHELLOPTS | grep xtrace` ]; then
		# Start bash with command tracing out to the log
		bash -x ${0} &> ${LOGPATH} || true
		poweroff
		exit 0;
	fi
fi

KERNELSRC=`ls ${SOURCEPATH}/linux-*.tar.*`

# Verify we have the name of a package
if [ "x${KERNELSRC}" = "x" ]; then
	echo "No kernel source found in ${SOURCEPATH}"
	exit 1
fi

# Look for one kernel package
if [ `echo ${KERNELSRC} | wc -l` -gt 1 ]; then
	echo "Too many kernel images:"
	echo $KERNELSRC
	exit 1
fi

# Look for the configuration
if [ ! -e "${CONFIGFILE}" ]; then
	echo "The configuration file ${CONFIGFILE} does not exist"
	exit 1
fi

cd /usr/src

extract_tar ${KERNELSRC}

# Get the kernel's name
KERNELNAME=`basename ${KERNELSRC} | sed -e 's/\.tar.*$//'`

if [ "x${MODULES}" != "x" ]; then
	for TAR in ${MODULES}; do
		extract_tar ${TAR}
	done
fi

# Should CD into the kernel's directory
cd ${KERNELNAME} 

cp ${CONFIGFILE} .config

if [ "x${ADDEDMODULES}" != "x" ]; then
	UNAME=`uname -r`
	mkdir -p /lib/modules/${UNAME}
	ln -s /usr/src/${KERNELNAME} /lib/modules/${UNAME}/build
	make-kpkg --append-to-version $APPENDTOVERSION modules_image kernel_image
else
	make-kpkg --append-to-version $APPENDTOVERSION kernel_image
fi

# clean up
cp /usr/src/*.deb ${DESTINATIONPATH}
cd /usr/src
rm -rf *
exit 0
